"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const object_graph_1 = require("./object-graph");
/*
    Binary types are platform-local (i.e. DOM and Node can have different ones)
 */
var BinaryTypeIdentifier;
(function (BinaryTypeIdentifier) {
    let constructors = {
        Blob: typeof Blob === "undefined" ? null : Blob,
        ArrayBuffer: typeof ArrayBuffer === "undefined" ? null : ArrayBuffer,
        ArrayBufferView: typeof Uint8Array === "undefined" ? null : (function () {
            let typedArrayProto = Object.getPrototypeOf(Uint8Array.prototype);
            return typedArrayProto.constructor;
        })(),
        Buffer: typeof Buffer === "undefined" ? null : Buffer
    };
    let measures = {
        Blob: (x) => x.size,
        ArrayBuffer: (x) => x.length,
        ArrayBufferView: (x) => x.length,
        Buffer: (x) => x.length
    };
    function getBinaryTypeInfo(binary) {
        if (!binary)
            return null;
        let myCtor = binary.constructor;
        for (let k of Object.keys(constructors)) {
            let ctor = constructors[k];
            if (ctor && binary instanceof ctor) {
                let size = measures[k](binary);
                return new object_graph_1.BinaryScalar(myCtor && myCtor.name || k, size);
            }
        }
        return null;
    }
    BinaryTypeIdentifier.getBinaryTypeInfo = getBinaryTypeInfo;
})(BinaryTypeIdentifier = exports.BinaryTypeIdentifier || (exports.BinaryTypeIdentifier = {}));
//# sourceMappingURL=binary-type-identifier.js.map