export interface NodeMetadata {
    depthExceeded?: boolean;
    sizeExceeded?: boolean;
}
export declare class NodeBase {
    metadata: NodeMetadata;
    withMetadata(metadata: NodeMetadata): this;
}
export declare class PropertyItem {
    name: string;
    value: Node;
    constructor(name: string, value: Node);
}
export declare class SparseArrayNode extends NodeBase {
    items: PropertyItem[];
    constructor(items: PropertyItem[]);
}
export declare class ArrayNode extends NodeBase {
    items: Node[];
    constructor(items: Node[]);
}
export declare class ObjectNode extends NodeBase {
    ctor: Function;
    readonly properties: PropertyItem[];
    constructor(ctor: Function, properties: PropertyItem[]);
}
export declare class TextBlockScalar extends NodeBase {
    lines: string[];
    constructor(lines: string[]);
}
export declare class EmptyObjectScalar extends NodeBase {
    readonly ctor: Function;
    constructor(ctor: Function);
}
export declare class CircularReferenceScalar extends NodeBase {
}
export declare class EmptyArrayScalar extends NodeBase {
}
export declare class UnresolvedGetterScalar extends NodeBase {
    constructor();
}
export declare class BinaryScalar extends NodeBase {
    name: string;
    size: number;
    constructor(name: string, size: number);
}
export declare class ThrewErrorScalar extends NodeBase {
    error: Error;
    constructor(error: Error);
}
export declare class DepthExceededScalar extends NodeBase {
    error: Error;
    constructor(error: Error);
}
export declare type LiteralScalar = boolean | string | number | Date | RegExp | null | undefined | Symbol | Function;
export declare type ScalarNode = LiteralScalar | ThrewErrorScalar | BinaryScalar | CircularReferenceScalar | EmptyArrayScalar | EmptyObjectScalar | UnresolvedGetterScalar;
export declare type ComplexNode = ObjectNode | SparseArrayNode | ArrayNode | TextBlockScalar;
export declare type Node = ScalarNode | ComplexNode;
