"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class YamprintFormatter {
    constructor() {
        this.arrayPrefix = `► `;
        this.circularReference = "~Circular~";
        this.emptyArray = "[]";
        this.arrayDepthExceeded = "[...] (depth exceeded)";
        this.sizeExceededToken = "... (size exceeded)";
        this.indent = "  ";
        this.unreoslvedGetter = "... (getter unresolved)";
        this.undefined = "undefined";
        this.null = "null";
    }
    _formatCtorTag(line) {
        return line && `|${line}|` || "";
    }
    emptyObject(ctor, metadata) {
        let str = "";
        if (metadata && metadata.depthExceeded) {
            if (ctor === Object)
                str = "{...}";
            else if (!ctor || !ctor.name)
                str = this._formatCtorTag("{|~anonymous~|}");
            else
                str = `{${this.constructorTag(ctor)}}`;
        }
        else {
            if (ctor === Object)
                str = "{}";
            else if (!ctor || !ctor.name)
                str = this._formatCtorTag("~anonymous~");
            else
                str = this.constructorTag(ctor);
        }
        return `${str} (depth exceeded)`;
    }
    propertyKey(key) {
        key = /[^a-zA-Z0-9_$]/.test(key) ? `'${key}'` : key;
        return `${key} = `;
    }
    theme(theme) {
        let self = this;
        class ThemedYampritFormatter extends YamprintFormatter {
            constructor(colors) {
                super();
                Object.keys(colors).forEach(k => {
                    if (typeof this[k] === "function") {
                        this[k] = function (...args) {
                            return colors[k].call(colors, self[k].apply(this, args));
                        };
                    }
                    else {
                        this[k] = colors[k].call(colors, this[k]);
                    }
                });
            }
        }
        return new ThemedYampritFormatter(theme);
    }
    constructorTag(ctor) {
        let line = "";
        if (ctor === Object || !ctor) {
            return "";
        }
        if (ctor.prototype instanceof Error) {
            let regularName = ctor.name;
            if (regularName === "Error" || !regularName) {
                regularName = ctor.name;
            }
            regularName = regularName || "Error";
            line += regularName;
            if (ctor.message) {
                line += `("${ctor.message}")`;
            }
        }
        else {
            line = ctor.name || "~anonymous~";
        }
        return this._formatCtorTag(line);
    }
    ;
    sparseArrayPrefix(index) {
        return `(${index}) ► `;
    }
    ;
    function(f) {
        let line = "function";
        if (f.name) {
            line += ` ${f.name}`;
        }
        return `|${line}|`;
    }
    binary({ name, size }) {
        let line = `|${name} : size = ${size}|`;
        return line;
    }
    threwAlert(err) {
        let line = "THREW";
        if (!(err instanceof Error)) {
            line += ` ${(err && err.toString || Object.prototype.toString).call(err)}`;
        }
        else {
            if (err.name) {
                line += ` ${err.name}`;
            }
            else if (err.constructor && err.constructor.name) {
                line += `${err.constructor.name}`;
            }
            else {
                line += " Error";
            }
            if (err.message) {
                line += `("${err.message}")`;
            }
        }
        return `‼ ${line} ‼`;
    }
    number(n) {
        return n.toString();
    }
    ;
    textBlock({ lines }) {
        for (let i = 0; i < lines.length; i++) {
            lines[i] = this.lineInMultilineBlock(lines[i]);
        }
        return lines;
    }
    lineInMultilineBlock(line) {
        return `| ${line}`;
    }
    string(s) {
        if (!s.match(/[(\r|\n)]/)) {
            return `'${s}'`;
        }
        else {
            return s;
        }
    }
    symbol(s) {
        return s.toString();
    }
    boolean(b) {
        return b.toString();
    }
    date(d) {
        return d.toString();
    }
    regexp(r) {
        return r.toString();
    }
}
exports.YamprintFormatter = YamprintFormatter;
//# sourceMappingURL=yamprint-formatter.js.map