"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const yamprint_formatter_1 = require("./yamprint-formatter");
const graph_builder_1 = require("./graph-builder");
const graph_printer_1 = require("./graph-printer");
function defaultsDeep2(base, ...objects) {
    if (!base) {
        base = {};
    }
    for (let obj of objects) {
        if (!obj)
            continue;
        Object.getOwnPropertyNames(obj).forEach(name => {
            if (!(name in base) || base[name] == null) {
                base[name] = obj[name];
            }
            else if (typeof base[name] === "object") {
                let myProto = Object.getPrototypeOf(base[name]);
                if (myProto === Object.prototype) {
                    defaultsDeep2(base[name], obj[name]);
                }
            }
        });
    }
    return base;
}
function create(options) {
    let options2 = defaultsDeep2({}, options);
    let { formatter } = options2;
    let builder = new graph_builder_1.YamprintGraphBuilder(options2);
    let printer = new graph_printer_1.YamprintGraphPrinter(formatter);
    let yamprinter = (obj) => {
        let graph = builder.toGraph(obj);
        return printer.print(graph);
    };
    yamprinter.extend = function (optionsSource) {
        let newOptions;
        if (optionsSource instanceof Function) {
            newOptions = optionsSource(options);
            newOptions = defaultsDeep2({}, newOptions, options);
        }
        else {
            newOptions = defaultsDeep2({}, optionsSource, options);
        }
        if (!(newOptions.formatter instanceof yamprint_formatter_1.YamprintFormatter)) {
            newOptions.formatter = options.formatter.theme(newOptions.formatter);
        }
        return create(newOptions);
    };
    return yamprinter;
}
exports.yamprint = create({
    formatter: new yamprint_formatter_1.YamprintFormatter(),
    propertyFilter: (prop) => {
        return ["constructor", "arguments", "prototype"].indexOf(prop.name) < 0 && !prop.name.startsWith("__") && !(prop.descriptor.value instanceof Function && prop.objectDepth > 0);
    },
    maxDepth: 10,
    maxObjectLength: 50,
    resolveGetters: true,
    isPrototypeExplorable: (proto) => {
        return [Error, Object, Function].map(x => x.prototype).indexOf(proto) < 0;
    },
    getConstructor: (o) => o.constructor
});
//# sourceMappingURL=yamprint.js.map